﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;
using Barsalt.WindowSystem;
using Barsalt.MessageServiceSystem;

namespace Barsalt.ConsoleSystem {
	class ConsoleWindow : BaseWindow {
		#region OBJECT_ATTRIBUTES
		//==========================================================================
		private static readonly string EMPTY				= "";
		private static readonly char[] SEPERATORS			= {' '};
		private string input								= "";
		private char commandSymbol							= '/';
		private Vector2 scrollPosition						= Vector2.zero;
		private bool isEnter								= false;
		private readonly List<Message> MESSAGES			= new List<Message>();
		private readonly Dictionary<LogType, Color> COLORS	= new Dictionary<LogType,Color>();
		private MessageServiceManager manager				= null;
		//==========================================================================
		#endregion

		#region CONSTRUCTOR
		//==========================================================================
		public ConsoleWindow(Rect dimension, string title) : base(dimension, title) {
			COLORS.Add(LogType.Log, Color.white);
			COLORS.Add(LogType.Exception, Color.red);
			COLORS.Add(LogType.Error, Color.red);
			COLORS.Add(LogType.Warning, Color.yellow);
			COLORS.Add(LogType.Assert, Color.cyan);
		}
		//==========================================================================
		#endregion

		#region OBJECT_FUNCTIONS
		//==========================================================================
		protected override void OnGui(int id) {
			scrollPosition = GUILayout.BeginScrollView(scrollPosition);

			for(int index = 0; index < MESSAGES.Count; index++){
				string message	= MESSAGES[index].Content;

				if(MESSAGES[index].StackTrace != null){
					message += "\n" + MESSAGES[index].StackTrace;
				}

				if(COLORS.ContainsKey(MESSAGES[index].Type)){
					GUI.contentColor = COLORS[MESSAGES[index].Type];
				}

				GUILayout.Label(message);

				GUI.contentColor = Color.white;
			}

			GUILayout.EndScrollView();

			input = GUILayout.TextArea(input);

			if(input.Length > 0 && input[input.Length - 1] == '\n')
				isEnter = true;

			input = Process(input);

			if(isEnter) {
				Message result = Execute(input);
				
				if(result != null){
					MESSAGES.Add(result);

					
				}

				scrollPosition	= new Vector2(0.0f, 100000.0f);

				input			= EMPTY;

				isEnter			= false;
			}
		}

		private string Process(string message){
			return message.Replace('\n', ' ');
		}

		private Message Execute(string message){
			if(message[0] == commandSymbol){
				message				= message.Substring(1);
				string[] elements	= message.Split(SEPERATORS);
				string[] parameters = new string[elements.Length - 1];

				for(int i = 1; i < elements.Length; i++) {
					parameters[i - 1] = elements[i];
				}

				return manager.ExecuteCommand(elements[0], parameters);
			} else {
				return new Message(LogType.Log, message);
			}
		}

		protected override void OnDimensionChange() {
			
		}

		public void HandleLog(string message, string stackTrace, LogType type) {
			MESSAGES.Add(new Message(type, message + "\n" + stackTrace));
		}

		public void HandleLog(Message message) {
			MESSAGES.Add(message);
		}
		//==========================================================================
		#endregion

		#region GETTER_AND_SETTER
		//==========================================================================
		public MessageServiceManager Manager{
			get{
				return manager;
			}
			set{
				manager = value;
			}
		}
		//==========================================================================
		#endregion
	}
}
