﻿using Barsalt.MessageServiceSystem;
using Barsalt.MessageServiceSystem.Services;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

namespace Barsalt.MessageServiceSystem {
	/// <summary>
	/// A manager were messages can be send to and services can register
	/// </summary>
	public sealed class MessageServiceManager{
		#region OBJECT_ATTRIBUTES
		//==========================================================================
		private readonly Dictionary<string, IMessageService> SERVICES	= new Dictionary<string, IMessageService>();
		private readonly List<IMessageService> SERVICE_LIST				= new List<IMessageService>();
		private IMessageService defaultService							= null;
		private string name												= "noname";
		//==========================================================================
		#endregion

		#region CONSTRUCTOR
		//==========================================================================
		public MessageServiceManager(){}
		//==========================================================================
		#endregion

		#region OBJECT_FUNCTIONS
		//==========================================================================
		/// <summary>
		/// receives messages, sends them to the appropriate service and returns the result
		/// </summary>
		/// <param name="message"></param>
		/// <returns></returns>
		public Message ExecuteCommand(string service, params string[] parameters){
			if(SERVICES.ContainsKey(service)){
				return SERVICES[service].ProcessMessage(parameters);
			} else if(defaultService != null){
				return defaultService.ProcessMessage(parameters);
			} else {
				return new Message(LogType.Error, "No service \"" + service + "\" found!");
			}
		}

		/// <summary>
		/// Add service
		/// </summary>
		/// <param name="name"></param>
		/// <param name="service"></param>
		public void Add(string name, IMessageService service) {
			if(!SERVICES.ContainsKey(name)){
				SERVICES.Add(name, service);

				SERVICE_LIST.Add(service);
			} else {
				throw new Exception("There is already a service registered under the name [" + name + "]");
			}
		}

		/// <summary>
		/// remove service
		/// </summary>
		/// <param name="name"></param>
		public void Remove(string name) {
			if(SERVICES.ContainsKey(name)) {
				SERVICE_LIST.Remove(SERVICES[name]);

				SERVICES.Remove(name);
			}
		}
		//==========================================================================
		#endregion

		#region GETTER_AND_SETTER
		//==========================================================================
		public IMessageService this[int index]{
			get{
				return SERVICE_LIST[index];
			}
		}

		public IMessageService this[string name]{
			get{
				return SERVICES[name];
			}
		}

		public int Count{
			get{
				return SERVICE_LIST.Count;
			}
		}

		public IMessageService DefaultService{
			get{
				return defaultService;
			}
			set{
				defaultService = value;
			}
		}

		public string Name{
			get{
				return name;
			}
			set{
				name = value;
			}
		}

		public string[] Services{
			get{
				return SERVICES.Keys.ToArray<string>();
			}
		}
		//==========================================================================
		#endregion
	}
}
